import phonenumbers
from django.test import SimpleTestCase, override_settings

from phonenumber_field.phonenumber import PhoneNumber


class PhoneNumberTest(SimpleTestCase):
    def test_phonenumber_default_format(self):
        phone = PhoneNumber.from_string("+33612345678")
        # E164 by default.
        self.assertEqual(str(phone), "+33612345678")
        with override_settings(PHONENUMBER_DEFAULT_FORMAT="E164"):
            self.assertEqual(str(phone), "+33612345678")
        with override_settings(PHONENUMBER_DEFAULT_FORMAT="INTERNATIONAL"):
            self.assertEqual(str(phone), "+33 6 12 34 56 78")
        with override_settings(PHONENUMBER_DEFAULT_FORMAT="NATIONAL"):
            self.assertEqual(str(phone), "06 12 34 56 78")
        with override_settings(PHONENUMBER_DEFAULT_FORMAT="RFC3966"):
            self.assertEqual(str(phone), "tel:+33-6-12-34-56-78")

    def test_phonenumber_extensions(self):
        for data in [
            "+33612345678 extension 456",
            "+33612345678 ext. 456",
            "+33612345678,456",
        ]:
            with self.subTest(data):
                phone = PhoneNumber.from_string(data)
                self.assertEqual(phone.as_international, "+33 6 12 34 56 78 ext. 456")
                self.assertEqual(phone.as_rfc3966, "tel:+33-6-12-34-56-78;ext=456")
                self.assertEqual(phone.as_national, "06 12 34 56 78 ext. 456")

    def test_phonenumber_formatting_properties(self):
        phone = PhoneNumber.from_string("+33612345678")
        self.assertEqual(phone.as_e164, "+33612345678")
        self.assertEqual(phone.as_international, "+33 6 12 34 56 78")
        self.assertEqual(phone.as_national, "06 12 34 56 78")
        self.assertEqual(phone.as_rfc3966, "tel:+33-6-12-34-56-78")


class PhoneNumberOrdering(SimpleTestCase):
    def test_ordering(self):
        phone1 = PhoneNumber.from_string("+33600000000")
        phone2 = PhoneNumber.from_string("+33600000001")
        self.assertLess(phone1, phone2)
        # Ordering is total.
        self.assertGreater(phone2, phone1)
        self.assertLessEqual(phone1, phone1)
        self.assertGreaterEqual(phone1, phone1)
        self.assertEqual(phone1, phone1)

    def test_ordering_with_phonenumbers(self):
        phone1 = PhoneNumber.from_string("+33600000000")
        phone2 = phonenumbers.parse("+33600000001")
        self.assertLess(phone1, phone2)

    def test_ordering_contains_None(self):
        phone1 = PhoneNumber.from_string("+33600000000")
        msg = "'<' not supported between instances of 'PhoneNumber' and 'NoneType'"
        for assertion in [
            self.assertLess,
            self.assertLessEqual,
            self.assertGreater,
            self.assertGreaterEqual,
        ]:
            with self.subTest(assertion):
                with self.assertRaisesMessage(TypeError, msg):
                    assertion(phone1, None)

    def test_ordering_with_invalid_value(self):
        phone1 = PhoneNumber.from_string("+33600000000")
        invalid = PhoneNumber.from_string("+1000000000")
        invalid_phonenumbers = phonenumbers.parse("+1000000000")
        for number in [invalid, invalid_phonenumbers]:
            with self.subTest(number):
                for p1, p2 in [[phone1, number], [number, phone1]]:
                    with self.subTest([p1, p2]):
                        with self.assertRaisesRegex(
                            ValueError, r"^Invalid phone number: "
                        ):
                            self.assertLess(p1, p2)



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html
import os
import pathlib
import sys

current_dir = pathlib.Path(__file__).parent
# Find current settings module for doctests.
sys.path.insert(0, str(current_dir))
sys.path.insert(0, str(current_dir / "ext"))

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "settings")

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = "django-phonenumber-field"
copyright = "2022, django-phonenumber-field"
author = "Stefan Foulis"

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.doctest",
    "sphinx.ext.intersphinx",
    "phonenumber_field_docs",
]

templates_path = ["_templates"]
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = "sphinx_rtd_theme"
html_static_path = []

# -- Options for extensions --------------------------------------------------

intersphinx_mapping = {
    "django": (
        "https://docs.djangoproject.com/en/dev/",
        "https://docs.djangoproject.com/en/dev/_objects/",
    ),
    "python": ("https://docs.python.org/3/", None),
}



from django.conf import settings
from django.core import checks
from django.db import models
from django.forms import Field
from django.utils.encoding import force_str
from django.utils.translation import gettext_lazy as _

from phonenumber_field import formfields
from phonenumber_field.phonenumber import PhoneNumber, to_python, validate_region
from phonenumber_field.validators import validate_international_phonenumber


class PhoneNumberDescriptor:
    """
    The descriptor for the phone number attribute on the model instance.
    Returns a PhoneNumber when accessed so you can do stuff like::

        >>> instance.phone_number.as_international

    Assigns a phone number object on assignment so you can do::

        >>> instance.phone_number = PhoneNumber(...)

    or,

        >>> instance.phone_number = '+414204242'
    """

    def __init__(self, field):
        self.field = field

    def __get__(self, instance, owner):
        if instance is None:
            return self

        # The instance dict contains whatever was originally assigned in
        # __set__.
        if self.field.name in instance.__dict__:
            value = instance.__dict__[self.field.name]
        else:
            instance.refresh_from_db(fields=[self.field.name])
            value = getattr(instance, self.field.name)
        return value

    def __set__(self, instance, value):
        instance.__dict__[self.field.name] = to_python(value, region=self.field.region)


class PhoneNumberField(models.CharField):
    attr_class = PhoneNumber
    descriptor_class = PhoneNumberDescriptor
    default_validators = [validate_international_phonenumber]

    description = _("Phone number")

    def __init__(self, *args, region=None, **kwargs):
        """
        :keyword str region: 2-letter country code as defined in ISO 3166-1.
            When not supplied, defaults to :setting:`PHONENUMBER_DEFAULT_REGION`
        :keyword int max_length: The maximum length of the underlying char field.
        """
        kwargs.setdefault("max_length", 128)
        super().__init__(*args, **kwargs)
        self._region = region

    @property
    def region(self):
        return self._region or getattr(settings, "PHONENUMBER_DEFAULT_REGION", None)

    def check(self, **kwargs):
        errors = super().check(**kwargs)
        errors.extend(self._check_region())
        return errors

    def _check_region(self):
        try:
            validate_region(self.region)
        except ValueError as e:
            return [checks.Error(force_str(e), obj=self)]
        return []

    def to_python(self, value):
        return to_python(value, region=self.region)

    def get_prep_value(self, value):
        """
        Perform preliminary non-db specific value checks and conversions.
        """
        parsed_value = super().get_prep_value(value)
        if not parsed_value:
            return parsed_value

        if parsed_value.is_valid():
            # A valid phone number. Normalize it for storage.
            format_string = getattr(settings, "PHONENUMBER_DB_FORMAT", "E164")
            fmt = PhoneNumber.format_map[format_string]
            value = parsed_value.format_as(fmt)
        else:
            # Not a valid phone number. Store the raw string.
            value = parsed_value.raw_input

        return value

    def from_db_value(self, value, expression, connection):
        return to_python(value)

    def contribute_to_class(self, cls, name, *args, **kwargs):
        super().contribute_to_class(cls, name, *args, **kwargs)
        setattr(cls, self.name, self.descriptor_class(self))

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        kwargs["region"] = self._region
        return name, path, args, kwargs

    def formfield(
        self,
        form_class=None,
        choices_form_class=None,
        **kwargs,
    ) -> Field | None:
        defaults = {
            "form_class": (
                formfields.PhoneNumberField if form_class is None else form_class
            ),
            "region": self.region,
            "error_messages": self.error_messages,
            "choices_form_class": choices_form_class,
        }
        defaults.update(kwargs)
        return super().formfield(**defaults)



from django.test import SimpleTestCase, override_settings

from phonenumber_field.phonenumber import PhoneNumber
from phonenumber_field.widgets import RegionalPhoneNumberWidget


class RegionalPhoneNumberWidgetTest(SimpleTestCase):
    @override_settings(PHONENUMBER_DEFAULT_FORMAT="INTERNATIONAL")
    def test_fallback_widget_switches_between_national_and_international(self):
        region = "GB"
        number_string = "01606 751 78"
        number = PhoneNumber.from_string(number_string, region=region)
        gb_widget = RegionalPhoneNumberWidget()
        gb_widget.region = "GB"
        de_widget = RegionalPhoneNumberWidget()
        de_widget.region = "DE"
        self.assertHTMLEqual(
            gb_widget.render("number", number),
            '<input name="number" type="tel" value="01606 75178" />',
        )
        self.assertHTMLEqual(
            de_widget.render("number", number),
            '<input name="number" type="tel" value="+44 1606 75178" />',
        )

        # If there's been a validation error, the value should be included verbatim
        self.assertHTMLEqual(
            gb_widget.render("number", "error"),
            '<input name="number" type="tel" value="error" />',
        )

    def test_fallback_defaults_to_phonenumber_default_format(self):
        region = "GB"
        number_string = "01606 751 78"
        number = PhoneNumber.from_string(number_string, region=region)
        widget = RegionalPhoneNumberWidget(region="FR")
        for fmt, expected in [
            ("INTERNATIONAL", "+44 1606 75178"),
            ("E164", "+44160675178"),
            ("RFC3966", "tel:+44-1606-75178"),
        ]:
            with self.subTest(fmt):
                with override_settings(PHONENUMBER_DEFAULT_FORMAT=fmt):
                    self.assertHTMLEqual(
                        widget.render("number", number),
                        f'<input name="number" type="tel" value="{expected}" />',
                    )

    def test_no_region(self):
        number_string = "+33612345678"
        number = PhoneNumber.from_string(number_string)
        widget = RegionalPhoneNumberWidget()
        for fmt, expected in [
            ("INTERNATIONAL", "+33 6 12 34 56 78"),
            ("E164", "+33612345678"),
            ("RFC3966", "tel:+33-6-12-34-56-78"),
        ]:
            with self.subTest(fmt):
                with override_settings(PHONENUMBER_DEFAULT_FORMAT=fmt):
                    self.assertHTMLEqual(
                        widget.render("number", number),
                        f'<input name="number" type="tel" value="{expected}" />',
                    )



from __future__ import annotations

from functools import total_ordering

import phonenumbers
from django.conf import settings
from django.core import validators


@total_ordering
class PhoneNumber(phonenumbers.PhoneNumber):
    """
    A extended version of phonenumbers.PhoneNumber that provides
    some neat and more pythonic, easy to access methods. This makes using a
    PhoneNumber instance much easier, especially in templates and such.
    """

    format_map = {
        "E164": phonenumbers.PhoneNumberFormat.E164,
        "INTERNATIONAL": phonenumbers.PhoneNumberFormat.INTERNATIONAL,
        "NATIONAL": phonenumbers.PhoneNumberFormat.NATIONAL,
        "RFC3966": phonenumbers.PhoneNumberFormat.RFC3966,
    }

    @classmethod
    def from_string(cls, phone_number, region=None):
        """
        :arg str phone_number: parse this :class:`str` as a phone number.
        :keyword str region: 2-letter country code as defined in ISO 3166-1.
            When not supplied, defaults to :setting:`PHONENUMBER_DEFAULT_REGION`
        """
        phone_number_obj = cls()
        if region is None:
            region = getattr(settings, "PHONENUMBER_DEFAULT_REGION", None)
        phonenumbers.parse(
            number=phone_number,
            region=region,
            keep_raw_input=True,
            numobj=phone_number_obj,
        )
        return phone_number_obj

    def __str__(self):
        if self.is_valid():
            format_string = getattr(settings, "PHONENUMBER_DEFAULT_FORMAT", "E164")
            fmt = self.format_map[format_string]
            return self.format_as(fmt)
        else:
            return self.raw_input

    def __repr__(self):
        if not self.is_valid():
            return f"Invalid{type(self).__name__}(raw_input={self.raw_input})"
        return super().__repr__()

    def is_valid(self):
        """
        Whether the number supplied is actually valid.

        :return: ``True`` when the phone number is valid.
        :rtype: bool
        """
        return phonenumbers.is_valid_number(self)

    def format_as(self, format):
        return phonenumbers.format_number(self, format)

    @property
    def as_international(self):
        return self.format_as(phonenumbers.PhoneNumberFormat.INTERNATIONAL)

    @property
    def as_e164(self):
        return self.format_as(phonenumbers.PhoneNumberFormat.E164)

    @property
    def as_national(self):
        return self.format_as(phonenumbers.PhoneNumberFormat.NATIONAL)

    @property
    def as_rfc3966(self):
        return self.format_as(phonenumbers.PhoneNumberFormat.RFC3966)

    def __len__(self):
        return len(str(self))

    def __eq__(self, other):
        """
        Override parent equality because we store only string representation
        of phone number, so we must compare only this string representation
        """
        if other in validators.EMPTY_VALUES:
            return False
        elif isinstance(other, str):
            default_region = getattr(settings, "PHONENUMBER_DEFAULT_REGION", None)
            other = to_python(other, region=default_region)
        elif isinstance(other, type(self)):
            # Nothing to do. Good to compare.
            pass
        elif isinstance(other, phonenumbers.PhoneNumber):
            # The parent class of PhoneNumber does not have .is_valid().
            # We need to make it match ours.
            old_other = other
            other = type(self)()
            other.merge_from(old_other)
        else:
            return False

        format_string = getattr(settings, "PHONENUMBER_DB_FORMAT", "E164")
        fmt = self.format_map[format_string]
        self_str = self.format_as(fmt) if self.is_valid() else self.raw_input
        other_str = other.format_as(fmt) if other.is_valid() else other.raw_input
        return self_str == other_str

    def __lt__(self, other):
        if isinstance(other, phonenumbers.PhoneNumber):
            old_other = other
            other = type(self)()
            other.merge_from(old_other)
        elif not isinstance(other, type(self)):
            raise TypeError(
                "'<' not supported between instances of "
                f"'{type(self).__name__}' and '{type(other).__name__}'"
            )

        invalid = None
        if not self.is_valid():
            invalid = self
        elif not other.is_valid():
            invalid = other
        if invalid is not None:
            raise ValueError(f"Invalid phone number: {invalid!r}")

        format_string = getattr(settings, "PHONENUMBER_DB_FORMAT", "E164")
        fmt = self.format_map[format_string]
        return self.format_as(fmt) < other.format_as(fmt)

    def __hash__(self):
        return hash(str(self))


def to_python(value, region=None):
    if value in validators.EMPTY_VALUES:  # None or ''
        phone_number = value
    elif isinstance(value, str):
        try:
            phone_number = PhoneNumber.from_string(phone_number=value, region=region)
        except phonenumbers.NumberParseException:
            # the string provided is not a valid PhoneNumber.
            phone_number = PhoneNumber(raw_input=value)
    elif isinstance(value, PhoneNumber):
        phone_number = value
    elif isinstance(value, phonenumbers.PhoneNumber):
        phone_number = PhoneNumber()
        phone_number.merge_from(value)
    else:
        raise TypeError(f"Can't convert {type(value).__name__} to PhoneNumber.")
    return phone_number


def validate_region(region):
    if region is not None and region not in phonenumbers.SUPPORTED_REGIONS:
        raise ValueError(
            f"“{region}” is not a valid region code. "
            f"Choices are {phonenumbers.SUPPORTED_REGIONS!r}"
        )



from unittest import mock

import django
import phonenumbers
from django import forms
from django.core.exceptions import ImproperlyConfigured
from django.test import SimpleTestCase, override_settings
from django.utils import translation
from django.utils.functional import lazy
from phonenumbers import COUNTRY_CODES_FOR_NON_GEO_REGIONS

from phonenumber_field.formfields import PhoneNumberField, SplitPhoneNumberField
from phonenumber_field.phonenumber import PhoneNumber
from phonenumber_field.validators import validate_phonenumber
from tests.models import (
    NullablePhoneNumber,
    PhoneNumberWithMaxLengthDefault,
    PhoneNumberWithMaxLengthNullable,
)

ALGERIAN_PHONE_NUMBER = "+213799136332"


class PhoneNumberFormFieldTest(SimpleTestCase):
    def test_error_message(self):
        class PhoneNumberForm(forms.Form):
            number = PhoneNumberField()

        form = PhoneNumberForm({"number": "invalid"})
        self.assertIs(form.is_valid(), False)
        self.assertEqual(
            form.errors, {"number": ["Enter a valid phone number (e.g. +12125552368)."]}
        )

    @override_settings(PHONENUMBER_DEFAULT_REGION="FR")
    def test_error_message_uses_default_region(self):
        class PhoneNumberForm(forms.Form):
            number = PhoneNumberField()

        form = PhoneNumberForm({"number": "invalid"})
        self.assertIs(form.is_valid(), False)
        self.assertEqual(
            form.errors,
            {
                "number": [
                    "Enter a valid phone number (e.g. 01 23 45 67 89) "
                    "or a number with an international call prefix."
                ]
            },
        )

    def test_override_error_message(self):
        class MyPhoneNumberField(PhoneNumberField):
            default_error_messages = {"invalid": "MY INVALID MESSAGE!"}

        class PhoneNumberForm(forms.Form):
            number = MyPhoneNumberField()

        form = PhoneNumberForm({"number": "invalid"})
        self.assertIs(form.is_valid(), False)
        self.assertEqual(form.errors, {"number": ["MY INVALID MESSAGE!"]})

    def test_override_error_message_inline(self):
        class PhoneNumberForm(forms.Form):
            number = PhoneNumberField(
                error_messages={"invalid": "MY INLINE INVALID MESSAGE!"}
            )

        form = PhoneNumberForm({"number": "invalid"})
        self.assertIs(form.is_valid(), False)
        self.assertEqual(form.errors, {"number": ["MY INLINE INVALID MESSAGE!"]})

    def test_algerian_phone_number_in_form(self):
        class PhoneNumberForm(forms.Form):
            number = PhoneNumberField()

        form = PhoneNumberForm({"number": ALGERIAN_PHONE_NUMBER})
        self.assertTrue(form.is_valid())
        self.assertEqual(ALGERIAN_PHONE_NUMBER, form.cleaned_data["number"])

    def test_error_message_lazy(self):
        def fail_gettext(msgid):
            raise Exception("gettext was called unexpectedly.")

        with mock.patch(
            "phonenumber_field.formfields.pgettext_lazy",
            side_effect=lazy(fail_gettext, str),
        ):
            PhoneNumberField()

    def test_override_widget(self):
        class MyPhoneNumberField(PhoneNumberField):
            widget = forms.TextInput

        class TestForm(forms.Form):
            phone = MyPhoneNumberField(region="FR")

        form = TestForm({"phone": "+33612345678"})
        self.assertIs(form.is_valid(), True)
        self.assertHTMLEqual(
            form.as_p(),
            """
            <p>
            <label for="id_phone">Phone:</label>
            <input id="id_phone" name="phone" required type="tel" value="+33612345678">
            </p>
            """,
        )

    @override_settings(PHONENUMBER_DEFAULT_REGION="FR")
    def test_widget_uses_default_region(self):
        class TestForm(forms.Form):
            phone = PhoneNumberField()

        form = TestForm({"phone": "+33612345678"})
        self.assertIs(form.is_valid(), True)
        self.assertHTMLEqual(
            form.as_p(),
            """
            <p>
            <label for="id_phone">Phone:</label>
            <input
               id="id_phone"
               name="phone"
               required
               type="tel"
               value="06 12 34 56 78">
            </p>
            """,
        )

    def test_invalid_phone_number(self):
        class PhoneNumberForm(forms.Form):
            number = PhoneNumberField()

        form = PhoneNumberForm({"number": "+11234567890"})
        self.assertIs(form.is_valid(), False)
        self.assertEqual(
            form.errors, {"number": ["Enter a valid phone number (e.g. +12125552368)."]}
        )

    def test_validate_shortcode(self):
        class ShortCodePhoneNumberField(PhoneNumberField):
            default_validators = [validate_phonenumber]

        class TestForm(forms.Form):
            phone = ShortCodePhoneNumberField(region="FR")

        form = TestForm({"phone": "1010"})
        self.assertIs(form.is_valid(), True)


class SplitPhoneNumberFormFieldTest(SimpleTestCase):
    def example_number(self, region_code: str) -> PhoneNumber:
        number = phonenumbers.example_number(region_code)
        e164 = phonenumbers.format_number(number, phonenumbers.PhoneNumberFormat.E164)
        return PhoneNumber.from_string(e164, region=region_code)

    def test_initial_empty(self):
        class TestForm(forms.Form):
            phone = SplitPhoneNumberField(region="CN")

        rendered = str(TestForm())
        self.assertIn('<option value="">---------</option>', rendered)
        self.assertIn('<option value="CN" selected>China +86</option>', rendered)

    @override_settings(PHONENUMBER_DEFAULT_REGION="CN")
    def test_uses_default_region_as_initial(self):
        class TestForm(forms.Form):
            phone = SplitPhoneNumberField()

        rendered = str(TestForm())
        self.assertIn('<option value="">---------</option>', rendered)
        self.assertIn('<option value="CN" selected>China +86</option>', rendered)

    def test_no_initial(self):
        class TestForm(forms.Form):
            phone = SplitPhoneNumberField()

        rendered = str(TestForm())
        self.assertIn('<option value="" selected>---------</option>', rendered)
        self.assertIn('<option value="CN">China +86</option>', rendered)
        for prefix in COUNTRY_CODES_FOR_NON_GEO_REGIONS:
            self.assertNotIn(f"+{prefix}", rendered)

    def test_initial(self):
        class TestForm(forms.Form):
            phone = SplitPhoneNumberField()

        rendered = str(
            TestForm(initial={"phone": PhoneNumber.from_string("+33612345678")})
        )
        self.assertIn('<option value="">---------</option>', rendered)
        self.assertIn('<option value="FR" selected>France +33</option>', rendered)
        self.assertIn(
            '<input type="tel" name="phone_1" value="612345678" '
            'required id="id_phone_1">',
            rendered,
        )

    def test_invalid_phone_number(self):
        class TestForm(forms.Form):
            phone = SplitPhoneNumberField()

        form = TestForm({"phone_0": "US", "phone_1": "1234567890"})
        self.assertIs(form.is_valid(), False)
        self.assertEqual(
            form.errors,
            {"phone": ["Enter a valid phone number (e.g. (201) 555-0123)."]},
        )

    @override_settings(USE_I18N=True)
    def test_after_translation_deactivate_all(self):
        class TestForm(forms.Form):
            phone = SplitPhoneNumberField()

        translation.deactivate_all()
        rendered = str(TestForm())
        self.assertIn(
            '<select name="phone_0" required id="id_phone_0">'
            '<option value="" selected>---------</option>',
            rendered,
        )

    def test_example_numbers(self):
        example_numbers = (
            self.example_number("US"),
            self.example_number("CA"),
            self.example_number("GB"),
            self.example_number("GG"),
            self.example_number("PL"),
            self.example_number("IT"),
        )
        tests = [
            (
                {
                    "phone_0": phonenumbers.region_code_for_number(number),
                    "phone_1": phonenumbers.national_significant_number(number),
                },
                number,
            )
            for number in example_numbers
        ]

        class TestForm(forms.Form):
            phone = SplitPhoneNumberField()

        for data, expected in tests:
            with self.subTest(data):
                form = TestForm(data=data)
                self.assertIs(form.is_valid(), True)
                self.assertEqual(form.cleaned_data, {"phone": expected})

    def test_empty_region(self):
        class TestForm(forms.Form):
            phone = SplitPhoneNumberField()

        invalid_national_number = "8876543210"
        form = TestForm(data={"phone_0": "", "phone_1": invalid_national_number})
        self.assertFalse(form.is_valid())
        rendered_form = form.as_ul()
        if django.VERSION >= (5, 2):
            self.assertInHTML(
                """<ul class="errorlist" id="id_phone_error">
                <li>This field is required.</li>
                </ul>
                """,
                rendered_form,
            )
        else:
            self.assertInHTML(
                '<ul class="errorlist"><li>This field is required.</li></ul>',
                rendered_form,
            )
        aria_invalid = "" if django.VERSION[0] < 5 else 'aria-invalid="true" '
        # Keeps national number input.
        self.assertInHTML(
            '<input type="tel" name="phone_1" '
            f'value="{invalid_national_number}" {aria_invalid} required '
            'id="id_phone_1">',
            rendered_form,
            count=1,
        )
        self.assertInHTML(
            '<option value="" selected>---------</option>',
            rendered_form,
            count=1,
        )

    def test_empty_national_number(self):
        class TestForm(forms.Form):
            phone = SplitPhoneNumberField()

        form = TestForm(data={"phone_0": "CA", "phone_1": ""})
        self.assertFalse(form.is_valid())
        rendered_form = form.as_ul()
        if django.VERSION >= (5, 2):
            self.assertInHTML(
                """
                <ul class="errorlist" id="id_phone_error">
                <li>This field is required.</li>
                </ul>
                """,
                rendered_form,
            )
        else:
            self.assertInHTML(
                '<ul class="errorlist"><li>This field is required.</li></ul>',
                rendered_form,
            )
        aria_invalid = "" if django.VERSION[0] < 5 else 'aria-invalid="true" '
        self.assertInHTML(
            f'<input type="tel" name="phone_1" {aria_invalid} '
            'required id="id_phone_1">',
            rendered_form,
            count=1,
        )

    def test_not_required_empty_data(self):
        class TestForm(forms.Form):
            phone = SplitPhoneNumberField(required=False)

        form = TestForm(data={"phone_0": "", "phone_1": ""})
        self.assertIs(form.is_valid(), True)
        self.assertEqual(form.cleaned_data["phone"], "")

    def test_no_region(self):
        class TestForm(forms.Form):
            phone = SplitPhoneNumberField()

        form = TestForm(data={"phone_1": "654321"})
        self.assertFalse(form.is_valid())
        rendered_form = form.as_ul()
        if django.VERSION >= (5, 2):
            self.assertInHTML(
                """
                <ul class="errorlist" id="id_phone_error">
                <li>This field is required.</li>
                </ul>
                """,
                rendered_form,
            )
        else:
            self.assertInHTML(
                '<ul class="errorlist"><li>This field is required.</li></ul>',
                rendered_form,
            )
        aria_invalid = "" if django.VERSION[0] < 5 else 'aria-invalid="true" '
        self.assertInHTML(
            f'<input type="tel" name="phone_1" value="654321" {aria_invalid} '
            'required id="id_phone_1">',
            rendered_form,
            count=1,
        )
        self.assertInHTML(
            '<option value="" selected>---------</option>',
            rendered_form,
            count=1,
        )

    def test_no_national_number(self):
        class TestForm(forms.Form):
            phone = SplitPhoneNumberField()

        form = TestForm(data={"phone_0": "CA"})
        self.assertFalse(form.is_valid())
        rendered_form = form.as_ul()
        if django.VERSION >= (5, 2):
            self.assertInHTML(
                """
                <ul class="errorlist" id="id_phone_error">
                <li>This field is required.</li>
                </ul>
                """,
                rendered_form,
                count=1,
            )
        else:
            self.assertInHTML(
                '<ul class="errorlist"><li>This field is required.</li></ul>',
                rendered_form,
                count=1,
            )
        aria_invalid = "" if django.VERSION[0] < 5 else 'aria-invalid="true" '
        self.assertInHTML(
            f'<input type="tel" name="phone_1" {aria_invalid} '
            'required id="id_phone_1">',
            rendered_form,
            count=1,
        )

    def test_not_required_no_data(self):
        class TestForm(forms.Form):
            phone = SplitPhoneNumberField(required=False)

        form = TestForm(data={})
        self.assertIs(form.is_valid(), True)
        self.assertEqual(form.cleaned_data["phone"], "")

    def test_not_required_empty_value(self):
        class TestForm(forms.Form):
            phone = SplitPhoneNumberField(required=False)
            phone_none = SplitPhoneNumberField(required=False, empty_value=None)

        form = TestForm(data={})
        self.assertIs(form.is_valid(), True)
        self.assertEqual(form.cleaned_data["phone"], "")
        self.assertIsNone(form.cleaned_data["phone_none"])

    def test_keeps_region_with_invalid_national_number(self):
        class TestForm(forms.Form):
            phone = SplitPhoneNumberField()

        form = TestForm(data={"phone_0": "CA", "phone_1": "0000"})
        self.assertFalse(form.is_valid())
        rendered_form = str(form)
        if django.VERSION >= (5, 2):
            self.assertInHTML(
                """
                <ul class="errorlist" id="id_phone_error">
                <li>Enter a valid phone number (e.g. (506) 234-5678).</li>
                </ul>
                """,
                rendered_form,
                count=1,
            )
        else:
            self.assertInHTML(
                '<ul class="errorlist">'
                "<li>Enter a valid phone number (e.g. (506) 234-5678).</li></ul>",
                rendered_form,
            )
        aria_invalid = "" if django.VERSION[0] < 5 else 'aria-invalid="true" '
        self.assertInHTML(
            f'<input type="tel" name="phone_1" value="0000" {aria_invalid} '
            'required id="id_phone_1">',
            rendered_form,
            count=1,
        )
        self.assertInHTML(
            '<option value="CA" selected>Canada +1</option>',
            rendered_form,
            count=1,
        )

    def test_maxlength_not_in_select(self):
        # Regression test for #490
        class TestForm(forms.Form):
            phone = SplitPhoneNumberField()

        rendered = str(TestForm())
        self.assertIn('<select name="phone_0" required id="id_phone_0">', rendered)

    def test_custom_attrs(self):
        class SplitPhoneNumberFieldWithClass(SplitPhoneNumberField):
            def prefix_field(self):
                prefix_field = super().prefix_field()
                prefix_field.widget.attrs["class"] = "prefix-input"
                return prefix_field

            def number_field(self):
                number_field = super().number_field()
                number_field.widget.attrs["class"] = "number-input"
                return number_field

        class TestForm(forms.Form):
            phone = SplitPhoneNumberFieldWithClass()

        rendered = str(TestForm())
        self.assertIn(
            '<select name="phone_0" class="prefix-input" required id="id_phone_0">',
            rendered,
        )
        self.assertInHTML(
            '<input type="tel" name="phone_1" class="number-input"'
            'required id="id_phone_1">',
            rendered,
            count=1,
        )

    def test_custom_choices(self):
        class SplitPhoneNumberFieldWithCountries(SplitPhoneNumberField):
            def prefix_field(self):
                return forms.ChoiceField(choices=[("FR", "France"), ("BE", "Belgium")])

        class TestForm(forms.Form):
            phone = SplitPhoneNumberFieldWithCountries()

        form = TestForm()
        if django.VERSION[0] < 4:
            self.assertHTMLEqual(
                str(form.as_p()),
                """
                <p>
                <label for="id_phone_0">Phone:</label>
                <select id="id_phone_0" name="phone_0" required>
                    <option value="FR">France</option>
                    <option value="BE">Belgium</option>
                </select>
                <input id="id_phone_1" name="phone_1" required type="tel">
                </p>
                """,
            )
        else:
            self.assertHTMLEqual(
                form.as_div(),
                """
                <div>
                <fieldset>
                <legend>
                Phone:
                </legend>
                <select id="id_phone_0" name="phone_0" required>
                    <option value="FR">France</option>
                    <option value="BE">Belgium</option>
                </select>
                <input id="id_phone_1" name="phone_1" required type="tel">
                </fieldset>
                </div>
                """,
            )

    def test_invalid_region(self):
        class PhoneNumberForm(forms.Form):
            number = SplitPhoneNumberField()

        form = PhoneNumberForm({"number_0": "NONEXISTENT", "number_1": "+33612345678"})
        self.assertIs(form.is_valid(), False)
        self.assertEqual(
            form.errors,
            {
                "number": [
                    "Select a valid choice. "
                    "NONEXISTENT is not one of the available choices."
                ]
            },
        )

    @override_settings(PHONENUMBER_DEFAULT_REGION="FR")
    def test_input_not_cleared_on_other_field_error(self):
        class PhoneNumberForm(forms.Form):
            name = forms.CharField(min_length=4, max_length=100)
            number = SplitPhoneNumberField(required=False)

        form = PhoneNumberForm({"name": "a", "number_0": "FR", "number_1": "612345678"})
        self.assertIs(form.is_valid(), False)
        self.assertEqual(
            form.errors,
            {"name": ["Ensure this value has at least 4 characters (it has 1)."]},
        )
        form_html = form.as_p()
        if django.VERSION >= (5, 2):
            self.assertInHTML(
                """
                <ul class="errorlist" id="id_name_error">
                    <li>
                    Ensure this value has at least 4 characters (it has 1).
                    </li>
                </ul>
                <p>
                    <label for="id_name">Name:</label>
                    <input
                       id="id_name"
                       maxlength="100"
                       minlength="4"
                       name="name"
                       aria-describedby="id_name_error"
                       aria-invalid="true"
                       required
                       type="text"
                       value="a">
                </p>
                """,
                form_html,
                count=1,
            )
        else:
            aria_invalid = "" if django.VERSION[0] < 5 else 'aria-invalid="true" '
            self.assertInHTML(
                f"""
                <ul class="errorlist">
                    <li>
                    Ensure this value has at least 4 characters (it has 1).
                    </li>
                </ul>
                <p>
                    <label for="id_name">Name:</label>
                    <input
                       id="id_name"
                       maxlength="100"
                       minlength="4"
                       name="name"
                       {aria_invalid}
                       required
                       type="text"
                       value="a">
                </p>
                """,
                form_html,
                count=1,
            )
        self.assertInHTML(
            '<option selected value="FR">France +33</option>',
            form_html,
            count=1,
        )
        self.assertInHTML(
            '<input id="id_number_1" name="number_1" type="tel" value="612345678">',
            form_html,
            count=1,
        )

    def test_without_babel(self):
        import phonenumber_field.formfields

        babel = phonenumber_field.formfields.babel

        def restore_babel():
            phonenumber_field.formfields.babel = babel

        self.addCleanup(restore_babel)
        phonenumber_field.formfields.babel = None
        with self.assertRaises(ImproperlyConfigured):
            SplitPhoneNumberField()

    def test_error_message_is_localized(self):
        class PhoneNumberForm(forms.Form):
            number = SplitPhoneNumberField()

        form = PhoneNumberForm({"number_0": "FR", "number_1": "1"})
        if django.VERSION >= (5, 2):
            self.assertIn(
                '<ul class="errorlist" id="id_number_error"><li>'
                "Enter a valid phone number (e.g. 01 23 45 67 89)."
                "</li></ul>",
                str(form),
            )
        else:
            self.assertIn(
                '<ul class="errorlist"><li>'
                "Enter a valid phone number (e.g. 01 23 45 67 89)."
                "</li></ul>",
                str(form),
            )

    def test_customize_invalid_error_message(self):
        class CustomSplitPhoneNumberField(SplitPhoneNumberField):
            def invalid_error_message(self):
                return "My message using {example_number}."

        class TestForm(forms.Form):
            phone = CustomSplitPhoneNumberField()

        form = TestForm({"phone_0": "FR", "phone_1": "1"})
        if django.VERSION >= (5, 2):
            self.assertInHTML(
                """
                <ul class="errorlist" id="id_phone_error">
                <li>My message using 01 23 45 67 89.</li>
                </ul>
                """,
                str(form),
                count=1,
            )
        else:
            self.assertIn(
                '<ul class="errorlist"><li>My message using 01 23 45 67 89.</li></ul>',
                str(form),
            )

    def test_clean_handles_invalid_input(self):
        data = [
            # require_all_fields, phone_data, error_message
            (True, {"phone_0": "", "phone_1": "1234"}, "This field is required."),
            (True, {}, "This field is required."),
            (True, {"phone_1": "1234"}, "This field is required."),
            (
                True,
                {"phone_0": "invalid", "phone_1": "1234"},
                "Select a valid choice. invalid is not one of the available choices.",
            ),
            (False, {"phone_0": "", "phone_1": "1234"}, "Enter a complete value."),
            (False, {}, "This field is required."),
            (False, {"phone_1": "1234"}, "Enter a complete value."),
            (
                False,
                {"phone_0": "invalid", "phone_1": "1234"},
                "Select a valid choice. invalid is not one of the available choices.",
            ),
        ]
        for all_fields_required, phone_data, error_message in data:
            with self.subTest(
                f"require_all_fields={all_fields_required},"
                f"{phone_data=},"
                f"{error_message=}"
            ):

                class TestForm(forms.Form):
                    phone = SplitPhoneNumberField(
                        require_all_fields=all_fields_required
                    )

                form = TestForm(phone_data)
                self.assertIs(form.is_valid(), False)
                self.assertEqual(form.errors["phone"], [error_message])

    def test_validate_shortcode(self):
        class ShortCodeSplitPhoneNumberField(SplitPhoneNumberField):
            default_validators = [validate_phonenumber]

        class TestForm(forms.Form):
            phone = ShortCodeSplitPhoneNumberField()

        form = TestForm({"phone_0": "FR", "phone_1": "1010"})
        self.assertIs(form.is_valid(), True)

    def test_formfield_with_maxlength_null(self):
        class TestForm(forms.Form):
            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)
                self.fields = forms.fields_for_model(
                    NullablePhoneNumber,
                    ["phone_number"],
                    field_classes={"phone_number": SplitPhoneNumberField},
                )

        form = TestForm(data={"phone_number_0": "FR", "phone_number_1": "612345678"})
        self.assertIs(form.is_valid(), True)
        self.assertEqual(form.cleaned_data["phone_number"], "+33612345678")

    def test_formfield_with_max_length_default(self):
        def make_form(model):
            class TestForm(forms.Form):
                def __init__(self, *args, **kwargs):
                    super().__init__(*args, **kwargs)
                    self.fields = forms.fields_for_model(
                        model,
                        ["phone"],
                        field_classes={"phone": SplitPhoneNumberField},
                    )

            return TestForm

        for model, empty_value in [
            (PhoneNumberWithMaxLengthDefault, ""),
            (PhoneNumberWithMaxLengthNullable, None),
        ]:
            TestForm = make_form(model)

            with self.subTest("no data", model=model):
                form = TestForm(data={})
                self.assertIs(form.is_valid(), True)
                self.assertEqual(form.cleaned_data["phone"], empty_value)

            with self.subTest("empty phone number", model=model):
                form = TestForm(data={"phone_0": "", "phone_1": ""})
                self.assertIs(form.is_valid(), True)
                self.assertEqual(form.cleaned_data["phone"], empty_value)

            with self.subTest("phone number within max_length", model=model):
                form = TestForm(data={"phone_0": "AU", "phone_1": "0444444444"})
                self.assertIs(form.is_valid(), True)
                self.assertEqual(form.cleaned_data["phone"], "+61444444444")

            with self.subTest("phone number exceeding max_length", model=model):
                form = TestForm(data={"phone_0": "FR", "phone_1": "33612345678"})
                self.assertIs(form.is_valid(), False)
                self.assertEqual(
                    form.errors,
                    {
                        "phone": [
                            "Ensure this value has no more than 12 characters, "
                            "“6 12 34 56 78” is 13 characters long."
                        ],
                    },
                )



from django.conf import settings
from django.utils.translation import gettext_lazy as _
from rest_framework import serializers
from rest_framework.exceptions import ValidationError

from phonenumber_field.phonenumber import PhoneNumber, to_python, validate_region


class PhoneNumberField(serializers.CharField):
    default_error_messages = {"invalid": _("Enter a valid phone number.")}

    def __init__(self, *args, region=None, **kwargs):
        """
        :keyword str region: 2-letter country code as defined in ISO 3166-1.
            When not supplied, defaults to :setting:`PHONENUMBER_DEFAULT_REGION`
        """
        super().__init__(*args, **kwargs)
        validate_region(region)
        self.region = region or getattr(settings, "PHONENUMBER_DEFAULT_REGION", None)

    def to_internal_value(self, data):
        if isinstance(data, PhoneNumber):
            phone_number = data
        else:
            str_value = super().to_internal_value(data)
            phone_number = to_python(str_value, region=self.region)

        if phone_number and not phone_number.is_valid():
            raise ValidationError(self.error_messages["invalid"])
        return phone_number



import django
import phonenumbers
from django import forms
from django.core import checks
from django.db.models import Model
from django.test import SimpleTestCase, TestCase, override_settings
from django.utils.encoding import force_str

from phonenumber_field import formfields, modelfields
from phonenumber_field.phonenumber import PhoneNumber, to_python

from . import models
from .forms import ARPhoneNumberForm, CustomPhoneNumberFormField, PhoneNumberForm

ALGERIAN_PHONE_NUMBER = "+213799136332"


def phone_transform(obj):
    return (obj.pk, obj.name, obj.phone)


class PhoneNumberFieldTestCase(TestCase):
    test_number_1 = "+4930123456"
    equal_number_strings = ["+44 113 8921113", "+441138921113", "tel:+44-113-892-1113"]
    local_numbers = [("GB", "01606 751 78"), ("DE", "0176/96842671")]
    storage_numbers = {
        "E164": ["+44 113 8921113", "+441138921113"],
        "RFC3966": ["+44 113 8921113", "tel:+44-113-892-1113"],
        "INTERNATIONAL": ["+44 113 8921113", "+44 113 892 1113"],
    }
    invalid_numbers = ["+44 113 892111"]

    def test_str_for_valid_number(self):
        self.assertEqual(
            str(PhoneNumber.from_string(self.test_number_1)),
            self.test_number_1,
        )

    def test_str_for_invalid_number(self):
        self.assertEqual(
            str(to_python("invalid")),
            "invalid",
        )

    def test_repr_for_invalid_number(self):
        self.assertEqual(
            repr(to_python("invalid")),
            "InvalidPhoneNumber(raw_input=invalid)",
        )

    def test_repr_for_valid_number(self):
        self.assertTrue(
            repr(PhoneNumber.from_string(self.test_number_1)).startswith(
                "PhoneNumber("
            ),
        )

    def test_valid_numbers_are_valid(self):
        numbers = [
            PhoneNumber.from_string(number_string)
            for number_string in self.equal_number_strings
        ]
        self.assertTrue(all(number.is_valid() for number in numbers))
        numbers = [
            PhoneNumber.from_string(number_string, region=region)
            for region, number_string in self.local_numbers
        ]
        self.assertTrue(all(number.is_valid() for number in numbers))

    def test_invalid_numbers_are_invalid(self):
        numbers = [
            PhoneNumber.from_string(number_string)
            for number_string in self.invalid_numbers
        ]
        self.assertTrue(all(not number.is_valid() for number in numbers))

    def test_same_invalid_numbers_are_equal(self):
        self.assertEqual(PhoneNumber(raw_input="invalid"), "invalid")

    def test_objects_with_same_number_are_equal(self):
        numbers = [
            models.MandatoryPhoneNumber.objects.create(
                phone_number=number_string
            ).phone_number
            for number_string in self.equal_number_strings
        ]
        self.assertTrue(
            all(
                phonenumbers.is_number_match(n, numbers[0])
                == phonenumbers.MatchType.EXACT_MATCH
                for n in numbers
            )
        )
        for number in numbers:
            self.assertEqual(number, numbers[0])
            for number_string in self.equal_number_strings:
                self.assertEqual(number, number_string)

    def test_same_number_has_same_hash(self):
        numbers = [
            PhoneNumber.from_string(number_string)
            for number_string in self.equal_number_strings
        ]
        numbers_set = set(numbers)
        self.assertEqual(len(numbers_set), 1)
        for number in numbers:
            self.assertIn(number, numbers_set)
        self.assertNotIn(self.test_number_1, numbers_set)

    def test_eq_and_ne(self):
        number_1 = "+411111111"
        number_2 = "+412222222"
        one = PhoneNumber.from_string("+411111111")
        two = PhoneNumber.from_string("+412222222")
        self.assertNotEqual(one, two)
        self.assertNotEqual(one, number_2)
        self.assertNotEqual(number_2, one)
        self.assertEqual(one, number_1)
        self.assertEqual(number_1, one)

    def test_blank_field_returns_empty_string(self):
        model = models.OptionalPhoneNumber()
        self.assertEqual(model.phone_number, "")
        model.phone_number = "+49 176 96842671"
        self.assertIsInstance(model.phone_number, PhoneNumber)

    def test_null_field_returns_none(self):
        model = models.NullablePhoneNumber()
        self.assertIsNone(model.phone_number)
        model.phone_number = self.test_number_1
        self.assertIsInstance(model.phone_number, PhoneNumber)
        model.phone_number = phonenumbers.parse(self.test_number_1, keep_raw_input=True)
        self.assertIsInstance(model.phone_number, PhoneNumber)

    def test_can_assign_string_phone_number(self):
        opt_phone = models.OptionalPhoneNumber()
        opt_phone.phone_number = self.test_number_1
        self.assertIsInstance(opt_phone.phone_number, PhoneNumber)
        self.assertEqual(opt_phone.phone_number.as_e164, self.test_number_1)
        opt_phone.full_clean()
        opt_phone.save()
        self.assertIsInstance(opt_phone.phone_number, PhoneNumber)
        self.assertEqual(opt_phone.phone_number.as_e164, self.test_number_1)
        opt_phone_db = models.OptionalPhoneNumber.objects.get(id=opt_phone.id)
        self.assertIsInstance(opt_phone_db.phone_number, PhoneNumber)
        self.assertEqual(opt_phone_db.phone_number.as_e164, self.test_number_1)

    def test_can_assign_phonenumber(self):
        """
        Tests assignment phonenumbers.PhoneNumber to field
        """
        opt_phone = models.OptionalPhoneNumber()
        opt_phone.phone_number = phonenumbers.parse(
            self.test_number_1, keep_raw_input=True
        )
        self.assertIsInstance(opt_phone.phone_number, PhoneNumber)
        self.assertEqual(opt_phone.phone_number.as_e164, self.test_number_1)
        opt_phone.full_clean()
        opt_phone.save()
        self.assertIsInstance(opt_phone.phone_number, PhoneNumber)
        self.assertEqual(opt_phone.phone_number.as_e164, self.test_number_1)
        opt_phone_db = models.OptionalPhoneNumber.objects.get(id=opt_phone.id)
        self.assertIsInstance(opt_phone_db.phone_number, PhoneNumber)
        self.assertEqual(opt_phone_db.phone_number.as_e164, self.test_number_1)

    def test_raise_on_invalid_values(self):
        msg = "Can't convert int to PhoneNumber."
        with self.assertRaisesMessage(TypeError, msg):
            to_python(42)

    def test_create_with_int(self):
        msg = "Can't convert int to PhoneNumber."
        with self.assertRaisesMessage(TypeError, msg):
            models.MandatoryPhoneNumber.objects.create(
                phone_number=int(self.test_number_1)
            )

    def _test_storage_formats(self):
        """
        Aggregate of tests to perform for db storage formats
        """
        self.test_objects_with_same_number_are_equal()
        self.test_blank_field_returns_empty_string()
        self.test_null_field_returns_none()
        self.test_can_assign_string_phone_number()

    @override_settings(PHONENUMBER_DEFAULT_REGION="DE")
    def test_storage_formats(self):
        """
        Perform aggregate tests for all db storage formats
        """
        for frmt in PhoneNumber.format_map:
            with override_settings(PHONENUMBER_DB_FORMAT=frmt):
                self._test_storage_formats()

    def test_prep_value(self):
        """
        Tests correct db storage value against different setting of
        PHONENUMBER_DB_FORMAT
        Required output format is set as string constant to guarantee
        consistent database storage values
        """
        number = modelfields.PhoneNumberField()
        for frmt in ["E164", "RFC3966", "INTERNATIONAL"]:
            with override_settings(PHONENUMBER_DB_FORMAT=frmt):
                self.assertEqual(
                    number.get_prep_value(to_python(self.storage_numbers[frmt][0])),
                    self.storage_numbers[frmt][1],
                )

    def test_phone_number_form_empty_value(self):
        form = PhoneNumberForm({"phone_number": ""})

        self.assertTrue(form.is_valid())
        self.assertDictEqual({"phone_number": None}, form.cleaned_data)

    def test_defer_phone_number_field(self):
        m = models.MandatoryPhoneNumber.objects.create(phone_number=self.test_number_1)
        m = models.MandatoryPhoneNumber.objects.defer("phone_number").get(pk=m.pk)
        self.assertEqual(m.phone_number, self.test_number_1)

    def test_filtering_by_invalid_number_does_not_raise_value_error(self):
        models.NullablePhoneNumber.objects.filter(phone_number="123")

    def test_filter_by_invalid_type_int_raises_value_error(self):
        msg = "Can't convert int to PhoneNumber"
        with self.assertRaisesMessage(TypeError, msg):
            models.NullablePhoneNumber.objects.filter(phone_number=123).exists()

    def test_filter_by_invalid_number_does_not_match_empty_or_none(self):
        models.NullablePhoneNumber.objects.create(phone_number="")
        models.NullablePhoneNumber.objects.create(phone_number=None)
        self.assertFalse(
            models.NullablePhoneNumber.objects.filter(phone_number="123").exists()
        )

    def test_filter_by_invalid_number_does_not_match_partial(self):
        models.NullablePhoneNumber.objects.create(phone_number="+4930123456")
        self.assertFalse(
            models.NullablePhoneNumber.objects.filter(phone_number="123").exists()
        )

    def test_filter_by_partial_number_of_valid_number_works(self):
        models.NullablePhoneNumber.objects.create(phone_number="+4923399999")
        models.NullablePhoneNumber.objects.create(phone_number="+4930123456")
        self.assertEqual(
            models.NullablePhoneNumber.objects.filter(
                phone_number__contains="9999"
            ).count(),
            1,
        )
        self.assertEqual(
            models.NullablePhoneNumber.objects.filter(
                phone_number__contains="123"
            ).count(),
            1,
        )

    def test_values_list(self):
        num0 = "+33600000000"
        num1 = "+33611111111"
        models.MandatoryPhoneNumber.objects.bulk_create(
            [
                models.MandatoryPhoneNumber(phone_number=num0),
                models.MandatoryPhoneNumber(phone_number=num1),
            ]
        )
        number0, number1 = models.MandatoryPhoneNumber.objects.order_by(
            "phone_number"
        ).values_list("phone_number", flat=True)
        self.assertIsInstance(number0, PhoneNumber)
        self.assertEqual(to_python(num0), number0)
        self.assertIsInstance(number1, PhoneNumber)
        self.assertEqual(to_python(num1), number1)

    def test_values_with_null(self):
        num0 = "+33600000000"
        models.NullablePhoneNumber.objects.bulk_create(
            [
                models.NullablePhoneNumber(phone_number=num0),
                models.NullablePhoneNumber(phone_number=""),
                models.NullablePhoneNumber(phone_number=None),
            ]
        )
        number0, number1, number2 = models.NullablePhoneNumber.objects.order_by(
            "pk"
        ).values_list("phone_number", flat=True)
        self.assertIsInstance(number0, PhoneNumber)
        self.assertEqual(to_python(num0), number0)
        self.assertIsInstance(number1, str)
        self.assertEqual(number1, "")
        self.assertIsNone(number2)


class PhoneNumberFieldAppTest(TestCase):
    def test_save_field_to_database(self):
        """Basic Field Test"""
        tm = models.TestModel()
        tm.phone = "+41 52 424 2424"
        tm.full_clean()
        tm.save()
        pk = tm.id

        tm = models.TestModel.objects.get(pk=pk)
        self.assertIsInstance(tm.phone, PhoneNumber)
        self.assertQuerySetEqual(
            models.TestModel.objects.all(),
            [(tm.pk, "", "+41524242424")],
            transform=phone_transform,
        )

    @override_settings(PHONENUMBER_DEFAULT_FORMAT="NATIONAL")
    def test_internal_phone_when_default_format_is_national(self):
        """Test that the region is not lost in a string conversion."""
        tm = models.TestModel()
        tm.phone = "+41 52 424 2424"
        tm.full_clean()
        tm.save()

        tm = models.TestModel.objects.get(pk=tm.pk)
        self.assertIsInstance(tm.phone, PhoneNumber)
        self.assertQuerySetEqual(
            models.TestModel.objects.all(),
            [(tm.pk, "", "+41524242424")],
            transform=phone_transform,
        )

    def test_create_with_invalid_number_works(self):
        obj = models.TestModel.objects.create(phone="invalid")
        self.assertEqual(obj.phone.raw_input, "invalid")
        obj = models.TestModel.objects.get(id=obj.id)
        self.assertEqual(obj.phone.raw_input, "invalid")

    def test_save_invalid_number_works(self):
        tm = models.TestModel.objects.create(phone="+1 604-333-4444")
        tm.phone = "invalid"
        tm.save()
        tm.refresh_from_db()
        self.assertEqual(tm.phone.raw_input, "invalid")

    def test_save_change_to_invalid_number_works(self):
        tm = models.TestModel.objects.create(phone="+1 604-333-4444")
        tm.phone = "invalid"
        tm.save(update_fields=["phone"])
        self.assertEqual(tm.phone.raw_input, "invalid")
        tm.refresh_from_db()
        self.assertEqual(tm.phone.raw_input, "invalid")

    def test_queryset_update_to_invalid_number_works(self):
        tm = models.TestModel.objects.create(phone="+1 604-333-4444")
        models.TestModel.objects.filter(id=tm.id).update(phone="invalid")
        tm.refresh_from_db()
        self.assertEqual(tm.phone.raw_input, "invalid")

    def test_save_blank_phone_to_database(self):
        """Field Test for when Blank"""
        tm = models.TestModelPhoneB()
        tm.save()

        pk = tm.id
        tm = models.TestModelPhoneB.objects.get(pk=pk)
        self.assertQuerySetEqual(
            models.TestModelPhoneB.objects.all(),
            [(tm.pk, "", "")],
            transform=phone_transform,
        )

    def __test_nullable_field_helper(self, TestModel):
        """Helper method for the next four tests."""
        tm = TestModel()
        tm.save()

        pk = tm.id
        tm = TestModel.objects.get(pk=pk)
        self.assertIsNone(tm.phone)
        self.assertQuerySetEqual(
            TestModel.objects.all(), [(tm.pk, "", None)], transform=phone_transform
        )

        # ensure that null values do not cause uniqueness conflicts
        TestModel.objects.create()
        self.assertEqual(TestModel.objects.count(), 2)

    def test_save_unique_null_phone_to_database(self):
        """Field Test for when Null & Unique"""
        self.__test_nullable_field_helper(models.TestModelPhoneNU)

    def test_save_unique_null_blank_phone_to_database(self):
        """Field Test for when Blank, Null & Unique"""
        self.__test_nullable_field_helper(models.TestModelPhoneBNU)

    def test_save_unique_null_none_phone_to_database(self):
        """Field Test for when No Default, Null & Unique"""
        self.__test_nullable_field_helper(models.TestModelPhoneNDNU)

    def test_save_unique_null_blank_none_phone_to_database(self):
        """Field Test for when Blank, No Default, Null & Unique"""
        self.__test_nullable_field_helper(models.TestModelPhoneBNDNU)

    def __test_nullable_default_field_helper(self, TestModel):
        """Helper method for the next two tests."""
        tm = TestModel()
        tm.save()

        pk = tm.id
        tm = TestModel.objects.get(pk=pk)
        self.assertQuerySetEqual(
            TestModel.objects.all(),
            [(tm.pk, "", "+41524242424")],
            transform=phone_transform,
        )

    def test_save_unique_null_default_phone_to_database(self):
        """Field Test for when Default, Null & Unique"""
        self.__test_nullable_default_field_helper(models.TestModelPhoneDNU)

    def test_save_unique_null_blank_default_phone_to_database(self):
        """Field Test for when Blank, Default, Null & Unique"""
        self.__test_nullable_default_field_helper(models.TestModelPhoneBDNU)

    def __test_nullable_empty_default_field_helper(self, TestModel):
        """Helper method for the next two tests."""
        tm = TestModel()
        tm.save()

        pk = tm.id
        tm = TestModel.objects.get(pk=pk)
        self.assertQuerySetEqual(
            TestModel.objects.all(), [(tm.pk, "", "")], transform=phone_transform
        )

    def test_save_unique_null_default_empty_phone_to_database(self):
        """Field Test for when Empty Default, Null & Unique"""
        self.__test_nullable_empty_default_field_helper(models.TestModelPhoneEDNU)

    def test_save_unique_null_blank_default_empty_phone_to_database(self):
        """Field Test for when Blank, Empty Default, Null & Unique"""
        self.__test_nullable_empty_default_field_helper(models.TestModelPhoneBEDNU)

    def test_model_attribute_can_be_accessed_on_class(self):
        self.assertIsInstance(models.TestModel.phone, modelfields.PhoneNumberDescriptor)


class RegionPhoneNumberFormFieldTest(SimpleTestCase):
    def test_regional_phone(self):
        class PhoneNumberForm(forms.Form):
            canadian_number = formfields.PhoneNumberField(region="CA")
            french_number = formfields.PhoneNumberField(region="FR")

        form = PhoneNumberForm(
            {"canadian_number": "604-686-2877", "french_number": "06 12 34 56 78"}
        )

        self.assertIs(form.is_valid(), True)
        self.assertEqual("+16046862877", form.cleaned_data["canadian_number"])
        self.assertEqual("+33612345678", form.cleaned_data["french_number"])

    def test_invalid_region(self):
        with self.assertRaises(ValueError) as cm:
            formfields.PhoneNumberField(region="invalid")

        self.assertTrue(
            force_str(cm.exception).startswith("“invalid” is not a valid region code.")
        )

    def test_error_message_nationalize_example(self):
        class PhoneNumberForm(forms.Form):
            number = formfields.PhoneNumberField(region="CA")

        form = PhoneNumberForm({"number": "invalid"})
        self.assertIs(form.is_valid(), False)
        self.assertEqual(
            form.errors,
            {
                "number": [
                    "Enter a valid phone number (e.g. (506) 234-5678) "
                    "or a number with an international call prefix."
                ]
            },
        )

    @override_settings(PHONENUMBER_DEFAULT_REGION="FR")
    def test_formfield_uses_default_region(self):
        class PhoneForm(forms.Form):
            phone_number = formfields.PhoneNumberField()

        form = PhoneForm()
        self.assertEqual("FR", form.fields["phone_number"].region)


class RegionPhoneNumberModelFieldTest(TestCase):
    def test_uses_region(self):
        m = models.FrenchPhoneOwner(cell_number="06 12 34 56 78")
        self.assertEqual(phonenumbers.parse("+33612345678"), m.cell_number)

    def test_accepts_international_numbers(self):
        num = "+16042234567"
        m = models.FrenchPhoneOwner(cell_number=num)
        self.assertEqual(phonenumbers.parse(num), m.cell_number)

    def test_formfield_uses_region(self):
        class FrenchPhoneForm(forms.ModelForm):
            class Meta:
                model = models.FrenchPhoneOwner
                fields = ["cell_number"]

        form = FrenchPhoneForm()
        self.assertEqual("FR", form.fields["cell_number"].region)

    def test_deconstruct_region(self):
        phone_field = modelfields.PhoneNumberField(region="CH")
        _name, path, args, kwargs = phone_field.deconstruct()
        self.assertEqual("phonenumber_field.modelfields.PhoneNumberField", path)
        self.assertEqual([], args)
        self.assertEqual({"max_length": 128, "region": "CH"}, kwargs)

    def test_deconstruct_no_region(self):
        phone_field = modelfields.PhoneNumberField()
        _name, path, args, kwargs = phone_field.deconstruct()
        self.assertEqual("phonenumber_field.modelfields.PhoneNumberField", path)
        self.assertEqual([], args)
        self.assertEqual({"max_length": 128, "region": None}, kwargs)

    def test_check_region(self):
        class InvalidRegionModel(Model):
            phone_field = modelfields.PhoneNumberField(region="invalid")

        errors = InvalidRegionModel.check()
        self.assertEqual(1, len(errors))
        error = errors[0]
        self.assertIsInstance(error, checks.Error)
        self.assertTrue(error.msg.startswith("“invalid” is not a valid region code."))
        self.assertEqual(error.obj, InvalidRegionModel._meta.get_field("phone_field"))

    def test_override_form_field(self):
        phone_number = models.CustomPhoneNumber()
        model_field = phone_number._meta.get_field("phone_number")
        self.assertIsInstance(model_field.formfield(), CustomPhoneNumberFormField)

    @override_settings(PHONENUMBER_DEFAULT_REGION="FR")
    def test_uses_default_region(self):
        m = models.MandatoryPhoneNumber.objects.create(phone_number="06 12 34 56 78")
        self.assertEqual(phonenumbers.parse("+33612345678"), m.phone_number)

    @override_settings(PHONENUMBER_DEFAULT_REGION="FR")
    def test_formfield_uses_default_region(self):
        class PhoneForm(forms.ModelForm):
            class Meta:
                model = models.MandatoryPhoneNumber
                fields = ["phone_number"]

        form = PhoneForm()
        self.assertEqual("FR", form.fields["phone_number"].region)

    @override_settings(PHONENUMBER_DEFAULT_REGION="FR")
    def test_desconstruct_does_not_use_default_region(self):
        field = modelfields.PhoneNumberField()
        _name, path, args, kwargs = field.deconstruct()
        self.assertEqual("phonenumber_field.modelfields.PhoneNumberField", path)
        self.assertEqual([], args)
        self.assertEqual({"max_length": 128, "region": None}, kwargs)

    @override_settings(PHONENUMBER_DEFAULT_REGION="FR")
    def test_region_uses_default_region(self):
        field = modelfields.PhoneNumberField()
        self.assertEqual("FR", field.region)

    def test_region_none(self):
        field = modelfields.PhoneNumberField()
        self.assertIsNone(field.region)

    def test_algerian_phone_number_in_model(self):
        m = models.MandatoryPhoneNumber.objects.create(
            phone_number=ALGERIAN_PHONE_NUMBER
        )
        self.assertEqual(phonenumbers.parse(ALGERIAN_PHONE_NUMBER), m.phone_number)
        self.assertEqual(ALGERIAN_PHONE_NUMBER, m.phone_number)

    def test_region_field_empty(self):
        obj = models.TestModelRegionAR.objects.create()
        form = ARPhoneNumberForm(instance=obj)
        self.assertHTMLEqual(
            form.as_p(),
            "<p>"
            '<label for="id_phone">Phone:</label>'
            "<input "
            'type="tel" '
            'name="phone" '
            'maxlength="128" '
            'id="id_phone">'
            "</p>",
        )

    def test_region_field_renders_international_as_E164(self):
        form = ARPhoneNumberForm({"phone": "+32468547825"})
        self.assertTrue(form.is_valid())
        self.assertHTMLEqual(
            form.as_p(),
            "<p>"
            '<label for="id_phone">Phone:</label>'
            "<input "
            'type="tel" '
            'name="phone" '
            'value="+32468547825" '
            'maxlength="128" '
            'id="id_phone">'
            "</p>",
        )

    def test_region_field_renders_international_as_E164_from_instance(self):
        obj = models.TestModelRegionAR.objects.create(phone="+32468547825")
        form = ARPhoneNumberForm(instance=obj)
        self.assertHTMLEqual(
            form.as_p(),
            "<p>"
            '<label for="id_phone">Phone:</label> '
            "<input "
            'type="tel" '
            'name="phone" '
            'value="+32468547825" '
            'maxlength="128" '
            'id="id_phone">'
            "</p>",
        )

    def test_region_field_renders_national_numbers_as_national(self):
        form = ARPhoneNumberForm({"phone": "01145482368"})
        self.assertTrue(form.is_valid())
        self.assertHTMLEqual(
            form.as_p(),
            "<p>"
            '<label for="id_phone">Phone:</label>'
            "<input "
            'type="tel" '
            'name="phone" '
            'value="011 4548-2368" '
            'maxlength="128" '
            'id="id_phone">'
            "</p>",
        )

    def test_region_field_renders_national_numbers_as_national_from_instance(self):
        obj = models.TestModelRegionAR.objects.create(phone="01145482368")
        form = ARPhoneNumberForm(instance=obj)
        self.assertHTMLEqual(
            form.as_p(),
            "<p>"
            '<label for="id_phone">Phone:</label> '
            "<input "
            'type="tel" '
            'name="phone" '
            'value="011 4548-2368" '
            'maxlength="128" '
            'id="id_phone">'
            "</p>",
        )

    def test_region_field_renders_national_numbers_from_instance_and_form_data(self):
        obj = models.TestModelRegionAR.objects.create(phone="01145482368")
        form = ARPhoneNumberForm({"phone": "011 4548-2368"}, instance=obj)
        self.assertTrue(form.is_valid())
        self.assertHTMLEqual(
            form.as_p(),
            "<p>"
            '<label for="id_phone">Phone:</label> '
            "<input "
            'type="tel" '
            'name="phone" '
            'value="011 4548-2368" '
            'maxlength="128" '
            'id="id_phone">'
            "</p>",
        )

    def test_region_field_renders_invalid_numbers(self):
        form = ARPhoneNumberForm({"phone": "abcdef"})
        self.assertFalse(form.is_valid())
        if django.VERSION >= (5, 2):
            self.assertHTMLEqual(
                form.as_p(),
                '<ul class="errorlist" id="id_phone_error">'
                "<li>Enter a valid phone number (e.g. 011 2345-6789) "
                "or a number with an international call prefix.</li>"
                "</ul>"
                "<p>"
                '<label for="id_phone">Phone:</label>'
                "<input "
                'type="tel" '
                'name="phone" '
                'value="abcdef" '
                'maxlength="128" '
                'aria-describedby="id_phone_error" '
                'aria-invalid="true" '
                'id="id_phone">'
                "</p>",
            )
        else:
            aria_invalid = "" if django.VERSION[0] < 5 else 'aria-invalid="true" '
            self.assertHTMLEqual(
                form.as_p(),
                '<ul class="errorlist">'
                "<li>Enter a valid phone number (e.g. 011 2345-6789) "
                "or a number with an international call prefix.</li>"
                "</ul>"
                "<p>"
                '<label for="id_phone">Phone:</label>'
                "<input "
                'type="tel" '
                'name="phone" '
                'value="abcdef" '
                'maxlength="128" '
                f"{aria_invalid}"
                'id="id_phone">'
                "</p>",
            )



from django.test import SimpleTestCase, override_settings
from rest_framework import renderers, serializers

from phonenumber_field.phonenumber import PhoneNumber
from phonenumber_field.serializerfields import PhoneNumberField

from .models import OptionalPhoneNumber


class PhoneNumberSerializerTest(SimpleTestCase):
    def test_blank_field(self):
        class PhoneNumberSerializer(serializers.ModelSerializer):
            class Meta:
                model = OptionalPhoneNumber
                fields = ["phone_number"]

        for data in [{}, {"phone": ""}]:
            with self.subTest(data):
                s = PhoneNumberSerializer(data=data)
                self.assertIs(s.is_valid(), True)
                self.assertEqual(s.data, {})

    def test_int(self):
        class PhoneNumberSerializer(serializers.Serializer):
            phone = PhoneNumberField()

        s = PhoneNumberSerializer(data={"phone": 1})
        self.assertIs(s.is_valid(), False)

    def test_empty_required(self):
        class PhoneNumberSerializer(serializers.Serializer):
            phone = PhoneNumberField()

        serializer = PhoneNumberSerializer(data={"phone": ""})
        self.assertIs(serializer.is_valid(), False)
        self.assertEqual(serializer.validated_data, {})

    def test_empty_optional(self):
        class PhoneNumberSerializer(serializers.Serializer):
            phone = PhoneNumberField(allow_blank=True)

        serializer = PhoneNumberSerializer(data={"phone": ""})
        self.assertIs(serializer.is_valid(), True)
        self.assertEqual(serializer.validated_data, {"phone": ""})

    def test_e164_phone_number(self):
        class PhoneNumberSerializer(serializers.Serializer):
            phone = PhoneNumberField()

        serializer = PhoneNumberSerializer(data={"phone": "+33612345678"})
        self.assertIs(serializer.is_valid(), True)
        self.assertEqual(serializer.validated_data, {"phone": "+33612345678"})
        self.assertIsInstance(serializer.validated_data["phone"], PhoneNumber)

    def test_region(self):
        class PhoneNumberSerializer(serializers.Serializer):
            phone = PhoneNumberField(region="FR")

        serializer = PhoneNumberSerializer(data={"phone": "0612345678"})
        self.assertIs(serializer.is_valid(), True)
        self.assertEqual(serializer.validated_data, {"phone": "+33612345678"})
        self.assertIsInstance(serializer.validated_data["phone"], PhoneNumber)

    def test_region_invalid(self):
        class PhoneNumberSerializer(serializers.Serializer):
            phone = PhoneNumberField(region="GB")

        serializer = PhoneNumberSerializer(data={"phone": "0612345678"})
        self.assertIs(serializer.is_valid(), False)
        self.assertEqual(serializer.validated_data, {})
        self.assertEqual(serializer.errors, {"phone": ["Enter a valid phone number."]})

    @override_settings(PHONENUMBER_DEFAULT_REGION="FR")
    def test_region_from_settings(self):
        class PhoneNumberSerializer(serializers.Serializer):
            phone = PhoneNumberField()

        serializer = PhoneNumberSerializer(data={"phone": "0612345678"})
        self.assertIs(serializer.is_valid(), True)
        self.assertEqual(serializer.validated_data, {"phone": "+33612345678"})
        self.assertIsInstance(serializer.validated_data["phone"], PhoneNumber)

    @override_settings(PHONENUMBER_DEFAULT_REGION="GB")
    def test_region_kwarg_precedes_setting(self):
        class PhoneNumberSerializer(serializers.Serializer):
            phone = PhoneNumberField(region="FR")

        serializer = PhoneNumberSerializer(data={"phone": "0612345678"})
        self.assertIs(serializer.is_valid(), True)
        self.assertEqual(serializer.validated_data, {"phone": "+33612345678"})
        self.assertIsInstance(serializer.validated_data["phone"], PhoneNumber)

    def test_invalid_region(self):
        with self.assertRaisesMessage(
            ValueError, "“INVALID” is not a valid region code. Choices are"
        ):

            class PhoneNumberSerializer(serializers.Serializer):
                phone = PhoneNumberField(region="INVALID")

    def test_multiple_isvalid_calls(self):
        class PhoneNumberSerializer(serializers.Serializer):
            phone = PhoneNumberField()

        serializer = PhoneNumberSerializer(data={"phone": "+33612345678"})
        self.assertIs(serializer.is_valid(), True)
        self.assertIs(serializer.is_valid(), True)

    def test_serialization(self):
        class PhoneNumberSerializer(serializers.Serializer):
            phone = PhoneNumberField(region="FR")

        for fmt, expected in [
            ("E164", b"+33612345678"),
            ("INTERNATIONAL", b"+33 6 12 34 56 78"),
            ("RFC3966", b"tel:+33-6-12-34-56-78"),
        ]:
            with override_settings(PHONENUMBER_DEFAULT_FORMAT=fmt), self.subTest(fmt):
                serializer = PhoneNumberSerializer(data={"phone": "0612345678"})
                serializer.is_valid()
                self.assertEqual(
                    b'{"phone":"%b"}' % expected,
                    renderers.JSONRenderer().render(serializer.data),
                )



import phonenumbers
from django.conf import settings
from django.core import validators
from django.core.exceptions import ImproperlyConfigured, ValidationError
from django.forms.fields import CharField, ChoiceField, MultiValueField
from django.utils import translation
from django.utils.text import format_lazy
from django.utils.translation import pgettext, pgettext_lazy
from phonenumbers import COUNTRY_CODE_TO_REGION_CODE, COUNTRY_CODES_FOR_NON_GEO_REGIONS

from phonenumber_field import widgets
from phonenumber_field.phonenumber import to_python, validate_region
from phonenumber_field.validators import validate_international_phonenumber

try:
    import babel
except ModuleNotFoundError:
    babel = None  # type: ignore

GEO_COUNTRY_CODE_TO_REGION_CODE = COUNTRY_CODE_TO_REGION_CODE.copy()
for country_code in COUNTRY_CODES_FOR_NON_GEO_REGIONS:
    del GEO_COUNTRY_CODE_TO_REGION_CODE[country_code]
# ISO 3166-1 alpha-2 to national prefix
REGION_CODE_TO_COUNTRY_CODE = {
    region_code: country_code
    for country_code, region_codes in GEO_COUNTRY_CODE_TO_REGION_CODE.items()
    for region_code in region_codes
}


class PhoneNumberField(CharField):
    default_validators = [validate_international_phonenumber]
    widget = widgets.RegionalPhoneNumberWidget

    def __init__(self, *args, region=None, widget=None, **kwargs):
        """
        :keyword str region: 2-letter country code as defined in ISO 3166-1.
            When not supplied, defaults to :setting:`PHONENUMBER_DEFAULT_REGION`
        :keyword django.forms.Widget widget: defaults to
            :class:`~phonenumber_field.widgets.RegionalPhoneNumberWidget`
        """
        validate_region(region)
        self.region = region or getattr(settings, "PHONENUMBER_DEFAULT_REGION", None)

        if widget is None:
            try:
                widget = self.widget(region=self.region)
            except TypeError:
                widget = self.widget()

        super().__init__(*args, widget=widget, **kwargs)
        self.widget.input_type = "tel"

        if "invalid" not in self.error_messages:
            if self.region:
                number = phonenumbers.example_number(self.region)
                example_number = to_python(number).as_national
                error_message = pgettext_lazy(
                    "{example_number} is a national phone number.",
                    "Enter a valid phone number (e.g. {example_number}) "
                    "or a number with an international call prefix.",
                )
            else:
                example_number = "+12125552368"  # Ghostbusters
                error_message = pgettext_lazy(
                    "{example_number} is an international phone number.",
                    "Enter a valid phone number (e.g. {example_number}).",
                )
            self.error_messages["invalid"] = format_lazy(
                error_message, example_number=example_number
            )

    def to_python(self, value):
        if value in validators.EMPTY_VALUES:
            return self.empty_value
        return to_python(value, region=self.region)


def localized_choices(language):
    if babel is None:
        raise ImproperlyConfigured(
            "The PhonePrefixSelect widget requires the babel package be installed."
        )
    choices = [("", "---------")]
    locale_name = translation.to_locale(language)
    locale = babel.Locale(locale_name)
    for region_code, country_code in REGION_CODE_TO_COUNTRY_CODE.items():
        region_name = locale.territories.get(region_code)
        if region_name:
            choices.append((region_code, f"{region_name} +{country_code}"))
    return choices


class PrefixChoiceField(ChoiceField):
    def __init__(self, *, choices=None, **kwargs):
        if choices is None:
            language = translation.get_language() or settings.LANGUAGE_CODE
            choices = localized_choices(language)
            choices.sort(key=lambda item: item[1])
        super().__init__(choices=choices, **kwargs)


class SplitPhoneNumberField(MultiValueField):
    default_validators = [validate_international_phonenumber]
    widget = widgets.PhoneNumberPrefixWidget

    def __init__(
        self,
        *,
        initial=None,
        region=None,
        widget=None,
        empty_value="",
        max_length=None,
        **kwargs,
    ):
        """
        :keyword list initial: A two-elements iterable:

            #. the region code, an :class:`str`, the 2-letter country code
               as defined in ISO 3166-1.

            #. the phone number, an :class:`str`

            When ``initial`` is not provided, the ``region`` keyword argument
            is used as the initial for the region field if specified, otherwise
            :setting:`PHONENUMBER_DEFAULT_REGION` is used.

            See :attr:`django.forms.Field.initial`.
        :keyword str region: 2-letter country code as defined in ISO 3166-1.
            When not supplied, defaults to :setting:`PHONENUMBER_DEFAULT_REGION`
        :keyword ~django.forms.MultiWidget widget: defaults to
            :class:`~phonenumber_field.widgets.PhoneNumberPrefixWidget`
        :keyword empty_value: value to use when the field is empty
        :keyword int max_length: maximum length of the phone number, when
            represented as :setting:`PHONENUMBER_DB_FORMAT`.
        """
        validate_region(region)
        region = region or getattr(settings, "PHONENUMBER_DEFAULT_REGION", None)
        if initial is None and region:
            initial = [region, None]
        prefix_field = self.prefix_field()
        number_field = self.number_field()
        fields = (prefix_field, number_field)
        if widget is None:
            widget = self.widget((prefix_field.widget, number_field.widget))
        self.empty_value = empty_value
        self.max_length = max_length
        super().__init__(fields, initial=initial, widget=widget, **kwargs)

    def prefix_field(self):
        """
        Returns the default :class:`~django.forms.Field` for the phone
        number prefix field.

        Use this hook to set widget attributes or update the field definition.
        """
        return PrefixChoiceField()

    def number_field(self):
        """
        Returns the default :class:`~django.forms.Field` for the phone
        number field.

        Use this hook to set widget attributes or update the field definition.
        """
        number_field = CharField()
        number_field.widget.input_type = "tel"
        return number_field

    def invalid_error_message(self):
        """
        Hook to customize ``error_messages["invalid"]`` for a given region.

        Include the example number in the message with the ``{example_number}``
        placeholder.
        """
        return pgettext(
            "{example_number} is a national phone number.",
            "Enter a valid phone number (e.g. {example_number}).",
        )

    def compress(self, data_list):
        if not data_list:
            return self.empty_value
        region, national_number = data_list
        return to_python(national_number, region=region)

    def clean(self, value):
        if not self.disabled:
            prefix_field = self.fields[0]
            try:
                region = prefix_field.clean(value[0])
            except ValidationError:
                pass  # The parent class handles validation.
            else:
                if region:
                    number = phonenumbers.example_number(region)
                    example_number = to_python(number).as_national
                    error_message = self.invalid_error_message()
                    self.error_messages["invalid"] = format_lazy(
                        error_message,
                        example_number=example_number,
                    )
        clean_value = super().clean(value)
        if clean_value not in self.empty_values and self.max_length is not None:
            phonenumber_str = clean_value.format_as(
                getattr(settings, "PHONENUMBER_DB_FORMAT", "E164")
            )
            if len(phonenumber_str) > self.max_length:
                raise ValidationError(
                    format_lazy(
                        "Ensure this value has no more than {max_length} characters, "
                        "“{value}” is {length} characters long.",
                        max_length=self.max_length,
                        value=phonenumber_str,
                        length=len(phonenumber_str),
                    )
                )
        return clean_value



import phonenumbers
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

from phonenumber_field.phonenumber import PhoneNumber, to_python


def validate_international_phonenumber(value):
    phone_number = to_python(value)
    if isinstance(phone_number, PhoneNumber) and not phone_number.is_valid():
        raise ValidationError(
            _("The phone number entered is not valid."), code="invalid"
        )


def validate_phonenumber(value):
    phone_number = to_python(value)
    if isinstance(phone_number, PhoneNumber) and not (
        phonenumbers.is_valid_number(phone_number)
        or phonenumbers.is_valid_short_number(phone_number)
    ):
        raise ValidationError(
            _("The phone number entered is not valid."), code="invalid"
        )



from django import forms

from phonenumber_field.formfields import PhoneNumberField

from .models import NullablePhoneNumber, TestModelRegionAR


class PhoneNumberForm(forms.ModelForm):
    class Meta:
        model = NullablePhoneNumber
        fields = ["phone_number"]


class CustomPhoneNumberFormField(PhoneNumberField):
    pass


class ARPhoneNumberForm(forms.ModelForm):
    class Meta:
        model = TestModelRegionAR
        fields = ["phone"]



from django.db import models

from phonenumber_field.modelfields import PhoneNumberField


class MandatoryPhoneNumber(models.Model):
    phone_number = PhoneNumberField()


class OptionalPhoneNumber(models.Model):
    phone_number = PhoneNumberField(blank=True)


class NullablePhoneNumber(models.Model):
    phone_number = PhoneNumberField(blank=True, null=True)


class CustomPhoneNumberModelField(PhoneNumberField):
    def formfield(self, **kwargs):
        from .forms import CustomPhoneNumberFormField

        return super().formfield(form_class=CustomPhoneNumberFormField)


class CustomPhoneNumber(models.Model):
    phone_number = CustomPhoneNumberModelField()


class TestModel(models.Model):
    """Basic Field Test"""

    name = models.CharField(max_length=255, blank=True)
    phone = PhoneNumberField()


class TestModelPhoneB(models.Model):
    """Field Test for when Blank"""

    name = models.CharField(max_length=255, blank=True)
    phone = PhoneNumberField(blank=True)


class TestModelPhoneNU(models.Model):
    """Field Test for when Null & Unique"""

    name = models.CharField(max_length=255, blank=True)
    phone = PhoneNumberField(null=True, unique=True)


class TestModelPhoneBNU(models.Model):
    """Field Test for when Blank, Null & Unique"""

    name = models.CharField(max_length=255, blank=True)
    phone = PhoneNumberField(blank=True, null=True, unique=True)


class TestModelPhoneNDNU(models.Model):
    """Field Test for when No Default, Null & Unique"""

    name = models.CharField(max_length=255, blank=True)
    phone = PhoneNumberField(default=models.NOT_PROVIDED, null=True, unique=True)


class TestModelPhoneBNDNU(models.Model):
    """Field Test for when Blank, No Default, Null & Unique"""

    name = models.CharField(max_length=255, blank=True)
    phone = PhoneNumberField(
        blank=True, default=models.NOT_PROVIDED, null=True, unique=True
    )


class TestModelPhoneDNU(models.Model):
    """Field Test for when Default, Null & Unique"""

    name = models.CharField(max_length=255, blank=True)
    phone = PhoneNumberField(default="+41524242424", null=True, unique=True)


class TestModelPhoneBDNU(models.Model):
    """Field Test for when Blank, Default, Null & Unique"""

    name = models.CharField(max_length=255, blank=True)
    phone = PhoneNumberField(blank=True, default="+41524242424", null=True, unique=True)


class TestModelPhoneEDNU(models.Model):
    """Field Test for when Empty Default, Null & Unique"""

    name = models.CharField(max_length=255, blank=True)
    phone = PhoneNumberField(default="", null=True, unique=True)


class TestModelPhoneBEDNU(models.Model):
    """Field Test for when Blank, Empty Default, Null & Unique"""

    name = models.CharField(max_length=255, blank=True)
    phone = PhoneNumberField(blank=True, default="", null=True, unique=True)


class FrenchPhoneOwner(models.Model):
    cell_number = PhoneNumberField(region="FR")


class TestModelRegionAR(models.Model):
    phone = PhoneNumberField(region="AR", blank=True, null=True)


class PhoneNumberWithMaxLengthDefault(models.Model):
    phone = PhoneNumberField(max_length=12, blank=True, default="")


class PhoneNumberWithMaxLengthNullable(models.Model):
    phone = PhoneNumberField(max_length=12, blank=True, null=True)



from django.conf import settings
from django.forms.widgets import MultiWidget, TextInput
from phonenumbers import (
    national_significant_number,
    region_code_for_number,
    region_codes_for_country_code,
)

from phonenumber_field.phonenumber import PhoneNumber, to_python


class PhoneNumberPrefixWidget(MultiWidget):
    """
    Companion widget of :class:`~phonenumber_field.formfields.SplitPhoneNumberField`.
    """

    def decompress(self, value):
        if isinstance(value, PhoneNumber):
            region_code = region_code_for_number(value)
            national_number = national_significant_number(value)
            return [region_code, national_number]
        return [None, None]


class RegionalPhoneNumberWidget(TextInput):
    """
    A :class:`~django.forms.Widget` that prefers displaying numbers in the
    national format, and falls back to :setting:`PHONENUMBER_DEFAULT_FORMAT`
    for international numbers.
    """

    input_type = "tel"

    def __init__(self, region=None, attrs=None):
        """
        :keyword str region: 2-letter country code as defined in ISO 3166-1.
            When not supplied, defaults to :setting:`PHONENUMBER_DEFAULT_REGION`
        :keyword dict attrs: See :attr:`django.forms.Widget.attrs`
        """
        if region is None:
            region = getattr(settings, "PHONENUMBER_DEFAULT_REGION", None)
        self.region = region
        super().__init__(attrs)

    def format_value(self, value):
        number = value
        if not isinstance(number, PhoneNumber):
            try:
                number = to_python(value, region=self.region)
            except TypeError:
                pass
        if isinstance(number, PhoneNumber) and number.country_code:
            region_codes = region_codes_for_country_code(number.country_code)
            if self.region in region_codes:
                return number.as_national
        return super().format_value(value)
